//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР  DHT22
/*
Описание для FLProg(параметры времени запуска определяются самим блоком FLProg).
---------------------------------------------------------------------------------------------------
DHT21,DHT22 - датчик температуры и влажности.
Поддерживается "горячее" подключение, ошибочные измерения отфильтровываются.
Основные параметры:
  1. Пользовательский номер пина[plug=OFF].
  2. Режим измерения - периодический[plug=ON], по переднему фронту eN 
                       или в каждом цикле[default=периодический].
  3. Пользовательский период измерения[plug=OFF,default=2200ms].
  4. Период измерения [default=2200ms].
Дополнительные параметры:
 1. "Количество повторов при ошибке измерения".
     После нескольких ошибочных измерений, выходные значения сбрасываются->
	                                              fresh=1;ok=0; T=-0.1; H=0.1. 
  2.Блокировка прерываний[default=OFF] - для контроллеров с внутренним работающим WiFi 
                                    снижает вероятность ошибочных измерений.
Входы:
  1. Номер пина [plug=ON].
  2. EN - разрешение работы блока [plug=OFF].
  3. eN - запуск измерения по переднему фронту [для режима измерения по переднему фронту].
Выходы:
  1. ok - флаг готовности устройства [plug=ON].
  2. T - температура (в градусах C).
  3. H - влажность (0-100%). 
  4. fresh - флаг готовности измерений(событие) [plug=OFF].
---------------------------------------------------------------------------------------------------
Особенности измерения: 
   Цикл измерения составляет менее 6мс. 
   В цикле измерения контроллер реагирует только на аппаратные прерывания, 
   время обработки которых не должны превышать 30-50мкс. 
   Большее время приводит к ошибочным измерениям. 
   Единичные ошибочные измерения (до qntMax default=3) отбрасываются.
---------------------------------------------------------------------------------------------------
Рекомендации для FLProg: 
1.Использовать "Диспетчер задач". 
2.Блок устанавливать на плате, управляемой "Диспетчером задач" как одна из задач
                                                 (медленная,быстрая фоновая и т.д.).
3.Если "Диспетчер задач" не используется, обеспечить регулярный вызов блока 
                                                    не реже периода измерения.
---------------------------------------------------------------------------------------------------
ecoins@mail.ru 01.10.2022. 
---------------------------------------------------------------------------------------------------

*/
// 1.Линия сенсора: притягивается к земле (600mсs); переводится в режим чтения; последовательно принимается 83 изменения линии.
//	 Если нет линия не меняется более 90mcs - фиксируется ошибка с кодом 2.
//    FALLING, RISING и FALLING - это стартовый бит, затем 40 бит: 
// 2. Проверяется контрольная сумма ---> codeErr=3.
// 3. Проверяется диапазон (температура -50.0/+125.0 градусов по Цельсию, влажность 0/100.0 %);
// 4. При отсутствии ошибок запоминаются полученные результаты и при необходимости взводится флаг изменившихся данных (id.fresh);
// 5. После  ошибочных чтений, превышающих maxQntErr, результатам T и H присваивается DHTx_NAN_INT
// Коды ошибок: 
// (=0)-нет ошибок;(=201)-пин не настроен(=255);(=202)-пин без функции DI;(=203) пин нет функции DO;
// (=100-184)-ошибка по тайм-ауту 90msc импульсов с данными;
// (=205)-ошибка контрольной суммы; (=206)-измеренные показания за пределами диапазонов измерений;
//  ecoins (ecoins@mail.ru) 
//  22.10.2022: Begin version
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
#ifndef RT_HW_DHT22_h
#define RT_HW_DHT22_h
#define RT_HW_DHT22_VER 			10		//--Номер версии библиотеки;
#include "RT_HW_BASE.h"						//--Подключение базовой библиотеки;
//-------------------------------------------------------------------------------------------------
#define RT_HW_DHT22_DELAY_ERR   	500 	//--Базовая задержка при повторов после ошибки;
#define RT_HW_DHT22_DELAY_ERR_MIN	500		//--Минимальный период при ошибках;
#define RT_HW_DHT22_DELAY_ERR_MAX	2500	//--Максимальный период при ошибках;
#define RT_HW_DHT22_TIME_SENSOR     1200	//--Время цикла измерения внутри сенсора после запуска измерения;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ DHT22
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_DHT22{
public:
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  custom=0;				//--Состояние настройки ID;  
//----------------Перенастраиваемые параметры пользователем----------------------------------------
uint8_t  pin=255;				//--Текущий номер пина;
//----------------Основные переменные управления измерением----------------------------------------
uint8_t  run=0, cmdRun=0;       //--Флаг для внешнего запуска измерений;
uint8_t  ddr=0;					//--Этапы основной программы управления устройствами;
uint8_t  step;					//--Управление этапами управления в функциях измерения;
//----------------Выходные параметры---------------------------------------------------------------
uint8_t  fresh=1;				//--Флаг "свежих" данных;
uint8_t  ok=0;					//--Флаг "Устройство в норме;
uint8_t  codeErr, err;			//--Код ошибки при обмене;
//----------------Дополнительные переменные управления измерением----------------------------------
uint8_t  initDevice, action;	//--Рабочие флаги;
uint8_t  cntErr;				//--Счетчик повторных ошибок;
//----------------Параметры для функци timeDelay()[и также используются функцией i2cFindAdr()]-----
uint32_t startDelay;		    //--Рабочий регистр;
uint8_t  workDelay=0;			//--Рабочий регистр; 
uint8_t  pointDelay;			//--Точка возврата из функции;
uint16_t sizeDelay;				//--Задержка для функции();
//----------------Буферные рабочие переменные (поддержка релевантности)----------------------------
uint8_t  blockInterrupts=0;     //--Блокировка прерываний при измерении;
uint8_t  modeDelay=1;			//--"Жесткая" задержка(=1) после импульса запуска;
uint32_t timeStart;
uint16_t rawH=0; uint16_t rawT=0; uint16_t data=0; uint32_t startTime; int16_t vT; uint16_t vH;
int8_t   idx;
uint8_t  age;
RT_HW_PIN_DIR_ID idPin;
float    vTemp, vHum;			//--Измеренные значения;
//=================================================================================================
void setParam(uint8_t vPin=255){ 	//++++НАСТРОЙКА параметров=====================================
if(custom==0){	//++++Однократная настройка параметров---------------------------------------------
	pin=vPin; custom=1;}	//--Настройка номера пина;
};	//+++++END setParam()==========================================================================

void direct(){						//++++УПРАВЛЕНИЕ измерением==================================== 
if(run){cmdRun=1;} 
//-------------------------------------------------------------------------------------------------
if(ddr==0) {	//++++Ожидание настройки-----------------------------------------------------------	
workDelay=ok=err=codeErr=cntErr=initDevice=0; fresh=1;	//--Инициализация параметров;
if(blockInterrupts){vTemp=-0.1; vHum=20.1;} else{vTemp=-0.4; vHum=30.4;}
            ddr=90; sizeDelay=50; pointDelay=10; 		//--Переход на инициализацию через 50ms;
}	//---------------------------------------------------------------------------------------------			  
if(ddr==10){	//++++Ожидание команды cmdRun(запуск измерения)------------------------------------
if(!cmdRun){return;} step=0;			//-- 
if(initDevice) {ddr=30;} else {ddr=15;}	//--Переход на измерение или на инициализацию;	
}	//---------------------------------------------------------------------------------------------
if(ddr==15){	//++++Проверка пина----------------------------------------------------------------
codeErr=0;
if(pin==255){codeErr=10;}															//--Выход, если пин не настроен;
if(codeErr==0){if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DIN_ID,pin)){codeErr=11;}	//--Выход по ошибке, если пин нe DI;
               if(!RT_HW_Base.checkPinPGM(RT_HW_PGM_PIN_DOT_ID,pin)){codeErr=12;}} 	//--Выход по ошибке, если пин не DO;
if(codeErr==0){digitalWrite(pin,1); pinMode(pin,OUTPUT); initDevice=1;   ddr=30;}	//--Настройка пина;
else                                                                    {ddr=10;}
}	//---------------------------------------------------------------------------------------------
if(ddr==30){	//++++Измерение--------------------------------------------------------------------
meter();		   						//--Измерение;
if(err)   {ddr=90; sizeDelay=3000; pointDelay=0;} 			//--Переход на начало через задержку 3000ms;														
if(action){return;}  										//--Ожидание завершения измерения;
cmdRun=0;  ddr=90; sizeDelay=500;  pointDelay=10; 			//--Переход на ожидание run через задержку 500ms;;
return;
}	//---------------------------------------------------------------------------------------------		
//-------------------------------------------------------------------------------------------------
if(ddr==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} ddr=pointDelay;
}	//---------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
} //--End direct();

void meter(){						//++++ИЗМЕРЕНИЕ параметров===================================== 
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------
workDelay=err=codeErr=cntErr=0; action=1; 					//--Очистка параметров;
rawH=0; rawT=0; data=0; timeStart=micros();
             step=10;										//--Переход на продолжение; 
}	//---------------------------------------------------------------------------------------------																
if(step==10){
digitalWrite(pin,LOW); pinMode(pin,OUTPUT); digitalWrite(pin,LOW); //--Притягивание линии устройства к земле;
// if(modeDelay){delayMicroseconds(RT_HW_DHT22_TIME_SENSOR);} 		//--"Жесткая" задержка;
//else  {if(RT_HW_Base.getPastMcs(timeStart)<RT_HW_DHT22_TIME_SENSOR){return;}}	//--"Мягкая"  задержка;
delayMicroseconds(RT_HW_DHT22_TIME_SENSOR);// Send start signal
digitalWrite(pin, HIGH);
pinMode(pin, INPUT); 							 		//--Настройка пина;
step=20;												//--Переход на измерение;
}	//--------------------------------------------------											
if(step==20){
RT_HW_PIN_DIR_SET_ID(idPin,pin);
//----------------------------------------------------------------------------------
// ESP32 is a multi core/multi processing chip.
// It is necessary to disable task switches during the readings
//----------------------------------------------------------------------------------
#if defined(RT_HW_CORE_ESP8266) || defined(RT_HW_CORE_ESP32)
if(!blockInterrupts){noInterrupts();}	
#endif
//----------------------------------------------------------------------------------
//	          Цикл фиксации 83 входных импульсов (первые три -стартовых импульса):
//1.cначала FALLING, RISING и FALLING фронт для стартового бита
//2.затем 40 бит: НАРАЩИВАЮЩИЙСЯ и затем СПАДАЮЩИЙ фронт на бит
//  Для упрощения кода принимаем любое значение HIGH или LOW, если оно не превышает 100 мкс.
//-----------------------------------------------------------------------------------			
for(idx=-3;idx<2*40; idx++){
	startTime = micros();
	do{age=(uint32_t)(micros()-startTime); if(age>100){codeErr=20+idx;	//--Проверка на тайм-аут;  	
#if defined(RT_HW_CORE_ESP8266) || defined(RT_HW_CORE_ESP32) || defined(RT_HW_CORE_ESP32_C3) 
if(!blockInterrupts){interrupts();}		//--Восстановление прерываний при ошибке;
#endif	   
                   step=85; pointDelay=10; return;}	//--Переход на динамическую задержку по тайм-ауту;
	   }while(RT_HW_PIN_DIR_READ(idPin)==(idx&1) ? HIGH : LOW); 
	if((idx>=0)&&(idx&1)){data<<=1;if(age>30){data|=1;}}
	if(idx==31){rawH=data;}								//--Фиксация данных по влажности;
	if(idx==63){rawT=data; data = 0;}					//--Фиксация данных по температуре;
	} //--Завершение цикла for(int8_t i=-3; i<80; i++)			 
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_ESP8266) || defined(RT_HW_CORE_ESP32) || defined(RT_HW_CORE_ESP32_C3) 
if(!blockInterrupts){interrupts();} //--Восстановление прерываний после завершения приема импульсов;
#endif
//---------------------------------------------------------------------------------
if((uint8_t)(((uint8_t)rawH)+(rawH>>8)+((uint8_t)rawT)+(rawT>>8))!= data){codeErr=200;}		//--Проверка на контрольную сумму;
if(codeErr==0){if(rawT & 0x8000 ){vT=-(int16_t)(rawT & 0x7FFF);} else {vT=rawT;} vH=rawH;}	//--Обработка результатов измерения;		       
if(codeErr==0){if(vH>1000){codeErr=201;}}													//--Проверка допустимости диапазона влажности;
if(codeErr==0){if(vT<-500){codeErr=202;} if(vT>1250){codeErr=203;}}							//--Проверка допустимости диапазона температуры;			   
if(codeErr==0){ok=fresh=1; vTemp=vT/10.0; vHum=vH/10.0;  action=0;  step=0; return;} 		//--Успешное завершение;
              step=85; pointDelay=10;														//--При ошибке переход на динамическую задержку;
}	//---------------------------------------------------------------------------------------------- 
//=================================================================================================
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>3){cntErr=3; codeErr=82; err=1; return;} sizeDelay=500+cntErr*500; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
};	//++++END meter()==============================================================================
//=================================================================================================
};	//++++END class RT_HW_DHT22====================================================================
#endif
//=================================================================================================
